//
// "gas" helper functions for TRT N-Tuple analysis framework
//

bool indet_isXenon(TRT *Trk, unsigned int ihit) { // HLP
  int gas = Trk->InDetTrackParticles_hit_gasType->at(ihit);
  if ( gas!=0 && gas!=1 ) ERROR("Unknown gas in indet_is_xenon().", gas);
  if ( gas==0 ) return true;
  return false;
}
bool indet_isArgon(TRT *Trk, unsigned int ihit) { // HLP
  int gas = Trk->InDetTrackParticles_hit_gasType->at(ihit);
  if ( gas!=0 && gas!=1 ) ERROR("Unknown gas in indet_is_argon().", gas);
  if ( gas==1 ) return true;
  return false;
}
bool gsf_isXenon(TRT *Trk, unsigned int ihit) { // HLP
  int gas = Trk->electron_gsfTrack_hit_gasType->at(ihit);
  if ( gas!=0 && gas!=1 ) ERROR("Unknown gas in gsf_is_xenon().", gas);
  if ( gas==0 ) return true;
  return false;
}
bool gsf_isArgon(TRT *Trk, unsigned int ihit) { // HLP
  int gas = Trk->electron_gsfTrack_hit_gasType->at(ihit);
  if ( gas!=0 && gas!=1 ) ERROR("Unknown gas in gsf_is_argon().", gas);
  if ( gas==1 ) return true;
  return false;
}

unsigned int dumpgas[2][34]={{0}}; // HLP
/* Gas geometry
   Insert  dump_gas_add(Trk);  inside your track loop
   Insert  dump_gas_show();    after your track loop
*/

void dump_gas_init() { // HLP
  for (int i=0; i<34; i++) {
    dumpgas[0][i]=0;
    dumpgas[1][i]=0;
  }
}

void dump_gas_increment(TRT *Trk) { // HLP
  /*
    Loop over the straws of this InDet track and increment
    the gas count for the (transformed) layer
  */
  for ( unsigned int ihit=0; ihit<Trk->InDetTrackParticles_hit_gasType->size(); ihit++ ) {

    int gas = Trk->InDetTrackParticles_hit_gasType->at(ihit);
    if ( gas!=0 && gas!=1 ) {
      WARN("Unknown gas in dump_gas_add().");
      continue;
    }

    int bec   = Trk->InDetTrackParticles_hit_bec->at(ihit);
    int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);

    int k=-1;
    if      ( bec==-2 ) k=13-layer;
    else if ( bec==-1 ) k=16-layer;
    else if ( bec==+1 ) k=17+layer;
    else if ( bec==+2 ) k=20+layer;
    else WARN("Bad bec value in dump_gas_add().");

    if ( k<0 || k>33 ) {
      WARN("Bad dumpgas index in dump_gas_add().");
      continue;
    }

    dumpgas[gas][k]++;

  } // end of straw loop
}

void dump_gas_show() { // HLP

  string color="";
  string element="";
  cout << CYellow << " Gas Geometry" << CNone << endl;
  //cout << " 3 2 1 0 9 8 7 6  5 4 3 2 1 0                     0 1 2 3 4 5  6 7 8 9 0 1 2 3" << endl;
  for ( unsigned int j=0; j<3; j++ ) {
    for ( unsigned int k=0; k<34; k++ ) {
      if  (dumpgas[0][k]>0 && dumpgas[1][k]>0)
        WARN("Array dumpgas has both gas types in the same module.");
      else if ( dumpgas[0][k]>0 ) color = CGreen;
      else if ( dumpgas[1][k]>0 ) color = CBlue;
      else                        color = CNone;
      if ( k<14 || k>19 ) {
        element="I";
        cout << " " << color << element;
      } else {
        element="========";
        if ( k==14 && j==0 ) cout << " " << color << element << CNone << CGrey << 2 << CNone;
        if ( k==15 && j==1 ) cout << " " << color << element << CNone << CGrey << 1 << CNone;
        if ( k==16 && j==2 ) cout << " " << color << element << CNone << CGrey << 0 << CNone;
        if ( k==17 && j==2 ) cout << color << element;
        if ( k==18 && j==1 ) cout << color << element;
        if ( k==19 && j==0 ) cout << color << element;
      }
      if ( k==7 || k==13 || k==19 || k==25 ) cout << " ";
    }
    cout << CNone << endl;
  }
  cout << CGrey << " 3 2 1 0 9 8 7 6  5 4 3 2 1 0                     0 1 2 3 4 5  6 7 8 9 0 1 2 3" << CNone <<endl;

  /* Run2 2016
    B0,B1
    EA3,EA5
    EC3,EC5,EC8
  */
  string s_run2_2016=" TrtStrawStatusHT-RUN2-scenario5-00-00; Argon: B0,B1, EA3,EA5, EC3,EC5,EC8; Used during 2016 data taking.";
  bool b_run2_2016=true;
  if ( !dumpgas[0][0]  ||  dumpgas[1][0] )  b_run2_2016=false;
  if ( !dumpgas[0][1]  ||  dumpgas[1][1] )  b_run2_2016=false;
  if ( !dumpgas[0][2]  ||  dumpgas[1][2] )  b_run2_2016=false;
  if ( !dumpgas[0][3]  ||  dumpgas[1][3] )  b_run2_2016=false;
  if ( !dumpgas[0][4]  ||  dumpgas[1][4] )  b_run2_2016=false;
  if (  dumpgas[0][5]  || !dumpgas[1][5] )  b_run2_2016=false;
  if ( !dumpgas[0][6]  ||  dumpgas[1][6] )  b_run2_2016=false;
  if ( !dumpgas[0][7]  ||  dumpgas[1][7] )  b_run2_2016=false;
  if (  dumpgas[0][8]  || !dumpgas[1][8] )  b_run2_2016=false;
  if ( !dumpgas[0][9]  ||  dumpgas[1][9] )  b_run2_2016=false;
  if (  dumpgas[0][10] || !dumpgas[1][10] ) b_run2_2016=false;
  if ( !dumpgas[0][11] ||  dumpgas[1][11] ) b_run2_2016=false;
  if ( !dumpgas[0][12] ||  dumpgas[1][12] ) b_run2_2016=false;
  if ( !dumpgas[0][13] ||  dumpgas[1][13] ) b_run2_2016=false;
  if ( !dumpgas[0][14] ||  dumpgas[1][14] ) b_run2_2016=false;
  if (  dumpgas[0][15] || !dumpgas[1][15] ) b_run2_2016=false;
  if (  dumpgas[0][16] || !dumpgas[1][16] ) b_run2_2016=false;
  if (  dumpgas[0][17] || !dumpgas[1][17] ) b_run2_2016=false;
  if (  dumpgas[0][18] || !dumpgas[1][18] ) b_run2_2016=false;
  if ( !dumpgas[0][19] ||  dumpgas[1][19] ) b_run2_2016=false;
  if ( !dumpgas[0][20] ||  dumpgas[1][20] ) b_run2_2016=false;
  if ( !dumpgas[0][21] ||  dumpgas[1][21] ) b_run2_2016=false;
  if ( !dumpgas[0][22] ||  dumpgas[1][22] ) b_run2_2016=false;
  if (  dumpgas[0][23] || !dumpgas[1][23] ) b_run2_2016=false;
  if ( !dumpgas[0][24] ||  dumpgas[1][24] ) b_run2_2016=false;
  if (  dumpgas[0][25] || !dumpgas[1][25] ) b_run2_2016=false;
  if ( !dumpgas[0][26] ||  dumpgas[1][26] ) b_run2_2016=false;
  if ( !dumpgas[0][27] ||  dumpgas[1][27] ) b_run2_2016=false;
  if ( !dumpgas[0][28] ||  dumpgas[1][28] ) b_run2_2016=false;
  if ( !dumpgas[0][29] ||  dumpgas[1][29] ) b_run2_2016=false;
  if ( !dumpgas[0][30] ||  dumpgas[1][30] ) b_run2_2016=false;
  if ( !dumpgas[0][31] ||  dumpgas[1][31] ) b_run2_2016=false;
  if ( !dumpgas[0][32] ||  dumpgas[1][32] ) b_run2_2016=false;
  if ( !dumpgas[0][33] ||  dumpgas[1][33] ) b_run2_2016=false;
  if ( b_run2_2016 ) cout << CGrey << s_run2_2016 << CNone << endl;

  /* Run2 2018
    B0,B1
    EA3,EA5
    EC2,EC5,EC8
  */
  string s_run2_2018=" TrtStrawStatusHT-MC-run2-scenario7_00-00; Argon: B0,B1, EA3,EA5, EC2,EC5,EC8; Used during 2017/2018 data taking.";
  bool b_run2_2018=true;
  if ( !dumpgas[0][0]  ||  dumpgas[1][0] )  b_run2_2018=false;
  if ( !dumpgas[0][1]  ||  dumpgas[1][1] )  b_run2_2018=false;
  if ( !dumpgas[0][2]  ||  dumpgas[1][2] )  b_run2_2018=false;
  if ( !dumpgas[0][3]  ||  dumpgas[1][3] )  b_run2_2018=false;
  if ( !dumpgas[0][4]  ||  dumpgas[1][4] )  b_run2_2018=false;
  if (  dumpgas[0][5]  || !dumpgas[1][5] )  b_run2_2018=false;
  if ( !dumpgas[0][6]  ||  dumpgas[1][6] )  b_run2_2018=false;
  if ( !dumpgas[0][7]  ||  dumpgas[1][7] )  b_run2_2018=false;
  if (  dumpgas[0][8]  || !dumpgas[1][8] )  b_run2_2018=false;
  if ( !dumpgas[0][9]  ||  dumpgas[1][9] )  b_run2_2018=false;
  if ( !dumpgas[0][10] ||  dumpgas[1][10] ) b_run2_2018=false;
  if (  dumpgas[0][11] || !dumpgas[1][11] ) b_run2_2018=false;
  if ( !dumpgas[0][12] ||  dumpgas[1][12] ) b_run2_2018=false;
  if ( !dumpgas[0][13] ||  dumpgas[1][13] ) b_run2_2018=false;
  if ( !dumpgas[0][14] ||  dumpgas[1][14] ) b_run2_2018=false;
  if (  dumpgas[0][15] || !dumpgas[1][15] ) b_run2_2018=false;
  if (  dumpgas[0][16] || !dumpgas[1][16] ) b_run2_2018=false;
  if (  dumpgas[0][17] || !dumpgas[1][17] ) b_run2_2018=false;
  if (  dumpgas[0][18] || !dumpgas[1][18] ) b_run2_2018=false;
  if ( !dumpgas[0][19] ||  dumpgas[1][19] ) b_run2_2018=false;
  if ( !dumpgas[0][20] ||  dumpgas[1][20] ) b_run2_2018=false;
  if ( !dumpgas[0][21] ||  dumpgas[1][21] ) b_run2_2018=false;
  if ( !dumpgas[0][22] ||  dumpgas[1][22] ) b_run2_2018=false;
  if (  dumpgas[0][23] || !dumpgas[1][23] ) b_run2_2018=false;
  if ( !dumpgas[0][24] ||  dumpgas[1][24] ) b_run2_2018=false;
  if (  dumpgas[0][25] || !dumpgas[1][25] ) b_run2_2018=false;
  if ( !dumpgas[0][26] ||  dumpgas[1][26] ) b_run2_2018=false;
  if ( !dumpgas[0][27] ||  dumpgas[1][27] ) b_run2_2018=false;
  if ( !dumpgas[0][28] ||  dumpgas[1][28] ) b_run2_2018=false;
  if ( !dumpgas[0][29] ||  dumpgas[1][29] ) b_run2_2018=false;
  if ( !dumpgas[0][30] ||  dumpgas[1][30] ) b_run2_2018=false;
  if ( !dumpgas[0][31] ||  dumpgas[1][31] ) b_run2_2018=false;
  if ( !dumpgas[0][32] ||  dumpgas[1][32] ) b_run2_2018=false;
  if ( !dumpgas[0][33] ||  dumpgas[1][33] ) b_run2_2018=false;
  if ( b_run2_2018 ) cout << CGrey << s_run2_2018 << CNone << endl;

  /* Run3 run numbers > 330000
    Full Barrel (B0,B1,B2)
    EA3,EA5,
    EC0,EC3,EC5,EC8,EC10
  */
  string s_run2run3=" TrtStrawStatusHT-MC-run2-run3-01; Run3 scenario run numbers 330000-inf: Full Barrel, EA3,EA5, EC0,EC3,EC5,EC8,EC10";
  bool b_run2run3=true;
  if ( !dumpgas[0][0]  ||  dumpgas[1][0] )  b_run2run3=false;
  if ( !dumpgas[0][1]  ||  dumpgas[1][1] )  b_run2run3=false;
  if ( !dumpgas[0][2]  ||  dumpgas[1][2] )  b_run2run3=false;
  if (  dumpgas[0][3]  || !dumpgas[1][3] )  b_run2run3=false;
  if ( !dumpgas[0][4]  ||  dumpgas[1][4] )  b_run2run3=false;
  if (  dumpgas[0][5]  || !dumpgas[1][5] )  b_run2run3=false;
  if ( !dumpgas[0][6]  ||  dumpgas[1][6] )  b_run2run3=false;
  if ( !dumpgas[0][7]  ||  dumpgas[1][7] )  b_run2run3=false;
  if (  dumpgas[0][8]  || !dumpgas[1][8] )  b_run2run3=false;
  if ( !dumpgas[0][9]  ||  dumpgas[1][9] )  b_run2run3=false;
  if (  dumpgas[0][10] || !dumpgas[1][10] ) b_run2run3=false;
  if ( !dumpgas[0][11] ||  dumpgas[1][11] ) b_run2run3=false;
  if ( !dumpgas[0][12] ||  dumpgas[1][12] ) b_run2run3=false;
  if (  dumpgas[0][13] || !dumpgas[1][13] ) b_run2run3=false;
  if (  dumpgas[0][14] || !dumpgas[1][14] ) b_run2run3=false;
  if (  dumpgas[0][15] || !dumpgas[1][15] ) b_run2run3=false;
  if (  dumpgas[0][16] || !dumpgas[1][16] ) b_run2run3=false;
  if (  dumpgas[0][17] || !dumpgas[1][17] ) b_run2run3=false;
  if (  dumpgas[0][18] || !dumpgas[1][18] ) b_run2run3=false;
  if (  dumpgas[0][19] || !dumpgas[1][19] ) b_run2run3=false;
  if ( !dumpgas[0][20] ||  dumpgas[1][20] ) b_run2run3=false;
  if ( !dumpgas[0][21] ||  dumpgas[1][21] ) b_run2run3=false;
  if ( !dumpgas[0][22] ||  dumpgas[1][22] ) b_run2run3=false;
  if (  dumpgas[0][23] || !dumpgas[1][23] ) b_run2run3=false;
  if ( !dumpgas[0][24] ||  dumpgas[1][24] ) b_run2run3=false;
  if (  dumpgas[0][25] || !dumpgas[1][25] ) b_run2run3=false;
  if ( !dumpgas[0][26] ||  dumpgas[1][26] ) b_run2run3=false;
  if ( !dumpgas[0][27] ||  dumpgas[1][27] ) b_run2run3=false;
  if ( !dumpgas[0][28] ||  dumpgas[1][28] ) b_run2run3=false;
  if ( !dumpgas[0][29] ||  dumpgas[1][29] ) b_run2run3=false;
  if ( !dumpgas[0][30] ||  dumpgas[1][30] ) b_run2run3=false;
  if ( !dumpgas[0][31] ||  dumpgas[1][31] ) b_run2run3=false;
  if ( !dumpgas[0][32] ||  dumpgas[1][32] ) b_run2run3=false;
  if ( !dumpgas[0][33] ||  dumpgas[1][33] ) b_run2run3=false;
  if ( b_run2run3 ) cout << CGrey << s_run2run3 << CNone << endl;

  cout << endl;

}
