//
// Helper functions for TRT N-Tuple analysis framework
// See helpers-hits.cpp for helpers relating to hit-level objects
//

// Colors!
const string CRed   ="\033[0;31m"; // HLP
const string CGreen ="\033[0;32m"; // HLP
const string CYellow="\033[0;33m"; // HLP
const string CBlue  ="\033[0;34m"; // HLP
const string CGrey  ="\033[0;90m"; // HLP
const string CNone  ="\033[0m"; // HLP

// Messaging
void INFO(string message) { // HLP
  cout << CGreen << " INFO: " << CNone << message << endl;
}
void INFO(string message, int val) { // HLP
  cout << CGreen << " INFO: " << CNone << message << " " << val << endl;
}
void INFO(string message, double val) { // HLP
  cout << CGreen << " INFO: " << CNone << message << " " << val << endl;
}
void WARN(string message) { // HLP
  cout << CYellow << " WARN: " << CNone << message << endl;
}
void WARN(string message, int val) { // HLP
  cout << CYellow << " WARN: " << CNone << message << " " << val << endl;
}
void WARN(string message, double val) { // HLP
  cout << CYellow << " WARN: " << CNone << message << " " << val << endl;
}
void ERROR(string message) { // HLP
  cout << CRed << " ERROR: " << CNone << message << endl;
  exit(1); // exit the program!
}
void ERROR(string message, int val) { // HLP
  cout << CRed << " ERROR: " << CNone << message << " " << val << endl;
  exit(1); // exit the program!
}
void ERROR(string message, double val) { // HLP
  cout << CRed << " ERROR: " << CNone << message << " " << val << endl;
  exit(1); // exit the program!
}

bool eventVeto(TRT *Trk) { // HLP
  bool veto=false;
  unsigned long thisevt = Trk->eventNumber;
  unsigned int thisrn = Trk->runNumber;
  if (thisevt==1648179242 && thisrn==430536) veto=true;
  return veto;
}

//
// Event Counting
//
unsigned long event_counter_previous_event_id=-1; // HLP
unsigned int  event_counter_n_events=0; // HLP

void event_counter_init() { // HLP
  event_counter_previous_event_id=-1;
  event_counter_n_events=0;
}

void event_counter_increment(TRT *Trk) { // HLP
  unsigned long event_id = Trk->eventNumber;
  if (event_id!=event_counter_previous_event_id) {
    event_counter_previous_event_id=event_id;
    event_counter_n_events++;
  }
}

void event_counter_show() { // HLP
  cout << CYellow << " EVNT: " << CNone << "Number of events = "
       << event_counter_n_events << endl;
}
//
// End Event counting
//


/*
  Track selection based on:
  - Tight Primary (https://twiki.cern.ch/twiki/bin/viewauth/AtlasProtected/TrackingCPRecsRun2R22)
  - pT at least 5 GeV/c
  - p < 50 GeV/c NO!, ONLY IF YOU WANT TO SUPPRESS MUON TRANSITION RADIATION
  - nTRT hits at least 15
*/
bool isSelectedTrack(TRT *Trk) { // HLP
  bool status=true;
  if ( !Trk->InDetTrackParticles_TightPrimary   ) status=false;
  if ( Trk->InDetTrackParticles_p4->Pt()<15000. ) status=false;
  //if ( Trk->InDetTrackParticles_p4->P()>50000.  ) status=false;
  if ( Trk->InDetTrackParticles_nTRTHits<15     ) status=false;
  return status;
}

bool goodProbe(TRT *Trk, bool input_sample_type) { // HLP
  // tnp-specific requirements.
  // You can add requirements for deltaZ0 and invariance mass.
  bool good=true;
  if (input_sample_type==0) { // 0=electron
    //double dzel = Trk->InDetTrackParticles_tnp_electron_deltaZ0;
    //double mel = Trk->InDetTrackParticles_tnp_electron_m*0.001;
    bool bel = Trk->InDetTrackParticles_tnp_electron;
    int qqel = Trk->InDetTrackParticles_tnp_electron_qxq;
    if (!bel) good=false;
    if (qqel!=-1) good=false;
  } else {          // 1=muon
    //double dzmu = Trk[sample]->InDetTrackParticles_tnp_muon_deltaZ0;
    //double mmu = Trk[sample]->InDetTrackParticles_tnp_muon_m*0.001;
    bool bmu = Trk->InDetTrackParticles_tnp_muon;
    int qqmu = Trk->InDetTrackParticles_tnp_muon_qxq;
    if (!bmu) good=false;
    if (qqmu!=-1) good=false;
  }
  return good;
}

int track_region(double eta) { // HLP
  // 0:NOTTRT; 1:Barrel; 2:Barrel-Endcap A transition; 3:ECA; 4:ECA-ECB transition; 5:ECB
  int track_region=0;
  int sign = eta<0 ? -1 : +1;
  double absEta = fabs(eta);
  if      ( absEta < 0.625 )  track_region=1;
  else if ( absEta < 1.070 )  track_region=2;
  else if ( absEta < 1.304 )  track_region=3;
  else if ( absEta < 1.752 )  track_region=4;
  else if ( absEta < 2.000 )  track_region=5;
  return sign*track_region;
}

int indet_track_region(TRT *Trk) { // HLP
  double eta = Trk->InDetTrackParticles_p4->Eta();
  return track_region(eta);
}

int gsf_track_region(TRT *Trk) { // HLP
  double eta = Trk->electron_gsfTrack_p4->Eta();
  return track_region(eta);
}

/*
  Track selection based on truth matching to a Z decay.
  For InDet tracks only (very little difference if you do this with GSF)
*/
bool hasTruthFromZ(TRT *Trk) { // HLP
  if ( Trk->InDetTrackParticles_truthMatchProbability>0.5 && Trk->truth_parentPdgId==23) return true;
  return false;
}

/*
  Implement https://twiki.cern.ch/twiki/pub/Atlas/TrtAnalysisRun2/iterGaus.C
  but with more control and checks.
*/
#include "TF1.h"
#include "TH1D.h"
void iterGaus(TH1D* h, double n_sigmas, double diff, unsigned int max_iter, bool print_it, double &mean, double &mean_err, double &sigma, double &sigma_err) { // HLP

  // Get full width at half maximum (FWHM) around the histogram peak
  int first_bin = h->FindFirstBinAbove(h->GetMaximum()/2);
  int  last_bin = h->FindLastBinAbove(h->GetMaximum()/2);

  if (first_bin==last_bin) {
    //AB WARN("Can't detect a peak in iterGaus(), returning zeros.");
    mean = 0.;
    mean_err = 0.;
    sigma = 0.;
    sigma_err = 0.;
    return;
  }

  double lower_bin_center = h->GetBinCenter(first_bin);
  double upper_bin_center = h->GetBinCenter(last_bin);
  double fwhm = upper_bin_center-lower_bin_center;

  // initialize the Gaussian mean and sigma
  mean  = (lower_bin_center+upper_bin_center)/2;
  sigma = fwhm/2.355;

  double previous_sigma = sigma;

  // Iterative a Gaussian fit
  if (print_it) cout << " iterGaus() " << 0
                     << "\t mean=" << mean
                     << "\t sigma=" << sigma << endl;

  unsigned int i=0;
  while(i<max_iter) {

    i++;

    if ( mean<lower_bin_center || mean>upper_bin_center ) {
      //AB WARN("iterGaus() mean going out of range! Reset to center. ", mean);
      mean = (lower_bin_center+upper_bin_center)/2;
    }

    h->Fit("gaus", "QRI", "",  mean-sigma*n_sigmas, mean+sigma*n_sigmas);
    mean      = h->GetFunction("gaus")->GetParameter(1);
    mean_err  = h->GetFunction("gaus")->GetParError(1);
    sigma     = h->GetFunction("gaus")->GetParameter(2);
    sigma_err = h->GetFunction("gaus")->GetParError(2);

    if (print_it) cout << " iterGaus() " << i
                       << "\t mean="  << mean  << "\t mean_err="  << mean_err
                       << "\t sigma=" << sigma << "\t sigma_err=" << sigma_err
                       << "\t sigma diff="  << sigma-previous_sigma << endl;

    if (fabs(sigma-previous_sigma)<diff) break;

    previous_sigma = sigma;

  }

  if (i==max_iter) {
    string mes = "Maximum iterations reached in iterGaus() for histogram ";
    mes = mes + h->GetName();
    WARN(mes.c_str(), (int)i);
  }

} // end of iterGaus
