/*
  TRT N-Tuple C++ analysis platform
  Original skeleton by Ahmet.Bingul@cern.ch September 2022
  Developments tracked at https://gitlab.cern.ch/atlas-trt-software/TRTnTupleSkeleton
  Compile: g++ skeleton.cpp -o skeleton.exe `root-config --cflags --glibs`
*/

#include "root_head.cpp"

//
// Main program
//
int main(int argc, char** argv) {

  if(argc!=4){
    cout << "Usage ./main [Run2/3] [Zee/Zmm] [true/false]\n";
    return 1;
  }

  //***********************************
  epoch  = argv[1];
  signal = argv[2];
  string rew = argv[3];
  if(rew=="true") reweighting = true;
  else reweighting = false;
  //epoch                 = "Run3";
  //signal                = "Zee";
  //reweighting           = false;
  //***********************************

  TRandom3 rng;
  double rwt[2][20] = {0.0};

  if(signal=="Zee" && epoch=="Run2"){
     #include "reweigthing_Run2Zee.h"
  }
  if(signal=="Zee" && epoch=="Run3"){
     #include "reweigthing_Run3Zee.h"
  }
  if(signal=="Zmm" && epoch=="Run2"){
     #include "reweigthing_Run2Zmm.h"
  }
  if(signal=="Zmm" && epoch=="Run3"){
     #include "reweigthing_Run3Zmm.h"
  }

  if(signal=="Zee") input_sample_type = 0;
  if(signal=="Zmm") input_sample_type = 1;

  if(reweighting)
    output_histogram_file = output_histogram_file + epoch + signal + "_reweighted.root";
  else
    output_histogram_file = output_histogram_file + epoch + signal + "_unweighted.root";

  if(epoch=="Run3" && signal=="Zee"){
    input_ntuple_file1 = "/eos/atlas/atlascerngroupdisk/det-indet/TRT/nTuples/tagAndProbe/group.det-indet.TRTNTupleTagAndProbe_Run3MC_1.0_ANALYSIS/group.det-indet.det-indet.601189.PhPy8EG_AZNLO_Zee.TRTxAOD.e8453_e8455_s3873_s3874_trt128-02_EXT0_EXT0.33154319.ANALYSIS.*.root";
    input_ntuple_file2 = "/eos/atlas/atlascerngroupdisk/det-indet/TRT/nTuples/tagAndProbe/group.det-indet.TRTNTupleTagAndProbe_data_v01_ANALYSIS/group.det-indet.data22_13p6TeV.*.merge.DRAW_EGZ.*.TRTxAOD_trt*_EXT0.32735619.ANALYSIS._*.root";
  }
  if(epoch=="Run3" && signal=="Zmm"){
    input_ntuple_file1 = "/eos/atlas/atlascerngroupdisk/det-indet/TRT/nTuples/tagAndProbe/group.det-indet.TRTNTupleTagAndProbe_Run3MC_1.0_ANALYSIS/group.det-indet.det-indet.601190.PhPy8EG_AZNLO_Zmumu.TRTxAOD.e8453_e8455_s3873_s3874_trt128-06_EXT0.33154319.ANALYSIS.*.root";
    input_ntuple_file2 = "/eos/atlas/atlascerngroupdisk/det-indet/TRT/nTuples/tagAndProbe/group.det-indet.TRTNTupleTagAndProbe_data_v01_ANALYSIS/group.det-indet.data22_13p6TeV.*.merge.DRAW_ZMUMU.*.TRTxAOD_*_EXT0.32735619.ANALYSIS._*.root";
  }
  if(epoch=="Run2" && signal=="Zee"){
    input_ntuple_file1 = "/eos/atlas/atlascerngroupdisk/det-indet/TRT/nTuples/tagAndProbe/group.det-indet.TRTNTupleTagAndProbe_mc_v02_ANALYSIS/group.det-indet.361106.PowhegPythia8EvtGen_AZNLOCTEQ6L1_Zee.TRTxAOD.e3601_e5984_s3126_s3136_trt125-06_EXT0.*.32738566.ANALYSIS._*.root";
    input_ntuple_file2 = "/eos/atlas/atlascerngroupdisk/det-indet/TRT/nTuples/tagAndProbe/group.det-indet.TRTNTupleTagAndProbe_data_v01_ANALYSIS/group.det-indet.data18_13TeV.00350144.merge.DRAW_EGZ.f933_m1831.TRTxAOD_trt125-01_EXT0.32735619.ANALYSIS._*.root";
  }
  if(epoch=="Run2" && signal=="Zmm"){
    input_ntuple_file1 = "/eos/atlas/atlascerngroupdisk/det-indet/TRT/nTuples/tagAndProbe/group.det-indet.TRTNTupleTagAndProbe_mc_v02_ANALYSIS/group.det-indet.361107.PowhegPythia8EvtGen_AZNLOCTEQ6L1_Zmumu.TRTxAOD.e3601_e5984_s3126_s3136_trt125-10_EXT0.*.32738566.ANALYSIS._*.root";
    input_ntuple_file2 = "/eos/atlas/atlascerngroupdisk/det-indet/TRT/nTuples/tagAndProbe/group.det-indet.TRTNTupleTagAndProbe_data_v01_ANALYSIS/group.det-indet.data18_13TeV.00350144.merge.DRAW_ZMUMU.f933_m1831.TRTxAOD_trt125-01_EXT0.32735619.ANALYSIS._*.root";
  }

  cout << "epoch                 = " << epoch << endl;
  cout << "signal                = " << signal << endl;
  cout << "reweighting           = " << reweighting << endl;
  cout << "output_histogram_file = " << output_histogram_file << endl;
  cout << "input_ntuple_file1    = " << input_ntuple_file1 << endl;
  cout << "input_ntuple_file2    = " << input_ntuple_file2 << endl;
  cout << endl;

  #include "root_book.cpp"

  // --- Data chain and tracks ---
  TChain *chain[2];

  chain[0] = new TChain("TRT","TRT N-Tuple");
  chain[0]->Add(input_ntuple_file1.c_str());
  chain[1] = new TChain("TRT","TRT N-Tuple");
  chain[1]->Add(input_ntuple_file2.c_str());

  TRT *Trk, *Track[2];
  Track[0] = new TRT(chain[0]);
  Track[1] = new TRT(chain[1]);

  #include "includes/dec_clock_start.cpp"

  // loop over samples
  for(int j=0; j<2; j++){
  Long64_t n_available_tracks = chain[j]->GetEntries();
  Long64_t n_tracks_to_loop_over = n_available_tracks;
  bool isData = j;
  Trk = Track[j];

  cout << "Sample "<< j << " is running " << endl;
  // === Track loop ======================================================
  for ( Long64_t itrk=0; itrk<n_tracks_to_loop_over; itrk++ ) {

    //#include "includes/dec_progress.cpp"
    Trk->GetEntry(itrk);

    // veto this event to avoid a crash
    if (Trk->eventNumber==1648179242 && Trk->runNumber==430536) continue;

    // uncomment to activate track selection criteria
    if ( !goodProbe(Trk,input_sample_type) ) continue;

    // Occupancy reweighting
    if(reweighting){
      double occupancy = Trk->InDetTrackParticles_trackOccupancy;
      unsigned int bin = int(20*occupancy);
      if (bin >= 20) bin = 20;
         // Randomly reject tracks such that the occupancy distribution of sample=0 (MC) is the same as that of sample=1 (DA)
         // This only works after you have updated the rwt[][] array.
         // To use this array you need to include "reweigthing.h" file.
         if(rng.Uniform() > rwt[isData][bin]) continue;
    }

    // global occupancy bin.
    int occ = int(10*Trk->InDetTrackParticles_trackOccupancy);
    if(occ>9) {occ =  9; cout << "occ index has been assigned to 9 since track occupancy is " << Trk->InDetTrackParticles_trackOccupancy << endl;}
    if(occ<0) {occ =  0; cout << "occ index has been assigned to 0 since track occupancy is " << Trk->InDetTrackParticles_trackOccupancy << endl;}

    // loop over InDet hits for this track
    unsigned int InDetHitVectorSize = indet_hit_vector_size(Trk);
    for ( unsigned int ihit=0; ihit<InDetHitVectorSize; ihit++ )
    {
       int    par       = indet_partition(Trk, ihit); // get partition number
       bool   gas       = indet_isArgon(Trk, ihit);   // 0=Xe, 1=Ar
       //bool   isht      = indet_isHT(Trk, ihit);
       int    bec       = Trk->InDetTrackParticles_hit_bec->at(ihit);
       //int    layer     = Trk->InDetTrackParticles_hit_layer->at(ihit);
       //double time      = Trk->InDetTrackParticles_hit_driftTime->at(ihit);
       //double rtow      = Trk->InDetTrackParticles_hit_rTrackToWire->at(ihit);
       bool   isPrecise = indet_isPrecise(Trk, ihit);
       //bool   isOutlier = indet_isOutlier(Trk, ihit);
       //bool   isTubeHit = indet_isTubehit(Trk, ihit);

       // convert partitionon number to array index
            if(par==-1) par = 0; // BA
       else if(par==+1) par = 1; // BC
       else if(par==+2) par = 2; // EAA
       else if(par==+3) par = 3; // EAB
       else if(par==-2) par = 4; // ECA
       else if(par==-3) par = 5; // ECB
       // convert barrel/endcap number to array index
            if(bec==+1) bec = 0; // BA
       else if(bec==-1) bec = 1; // BC
       else if(bec==+2) bec = 2; // EA
       else if(bec==-2) bec = 3; // EC

       if( !isPrecise ) continue;

       double unbiasedResidual = Trk->InDetTrackParticles_hit_unbiasedResidual->at(ihit);

       hResidual[par][gas][occ][j]->Fill(unbiasedResidual); // TH1D object
    } // end of hit loop

  } // === END Track loop ====================================================
  } // end of sample loop

  #include "includes/dec_clock_stop.cpp"

  // Write all histograms and close the file (do this at the end).
  h_out->Write();
  h_out->Close();

  cout << "All OK." << endl;
  return 0;
}
// EOF
